@@ -18,101 +18,25 @@
 """Utility functions for dealing with control files."""
 
 import collections
-from io import BytesIO
+from itertools import takewhile
 
 from debian.changelog import Version
-from debian.deb822 import Deb822
 
 from ._deb822 import PkgRelation
-
-from .reformatting import (
-    check_generated_file,
-    edit_formatted_file,
-    )
+from .deb822 import update_deb822
 
 
-class FormattingUnpreservable(Exception):
-    """Formatting unpreservable."""
-
-
-def dump_paragraphs(paragraphs):
-    """Dump a set of deb822 paragraphs to a file.
-
-    Args:
-      paragraphs: iterable over paragraphs
-    Returns:
-      formatted text (as bytes)
-    """
-    outf = BytesIO()
-    first = True
-    for paragraph in paragraphs:
-        if paragraph:
-            if not first:
-                outf.write(b'\n')
-            paragraph.dump(fd=outf, encoding='utf-8')
-            first = False
-    return outf.getvalue()
-
-
-def reformat_deb822(contents):
-    """Check whether it's possible to preserve a control file.
-
-    Args:
-      contents: Original contents
-    Returns:
-      New contents
-    """
-    return dump_paragraphs(
-        Deb822.iter_paragraphs(BytesIO(contents), encoding='utf-8'))
-
-
-def update_control(path='debian/control', **kwargs):
-    """Update a control file.
-
-    The callbacks can modify the paragraphs in place, and can trigger their
-    removal by clearing the paragraph.
-
-    Args:
-      path: Path to the debian/control file to edit
-      source_package_cb: Called on source package paragraph
-      binary_package_cb: Called on each binary package paragraph
-    Returns:
-      boolean indicating whether any changes were made
-    """
-    check_generated_file(path)
-    with open(path, 'rb') as f:
-        original_contents = f.read()
-    rewritten_contents = reformat_deb822(original_contents)
-    outf = BytesIO()
-    update_control_file(BytesIO(original_contents), outf, **kwargs)
-    updated_contents = outf.getvalue()
-    return edit_formatted_file(
-        path, original_contents, rewritten_contents, updated_contents)
-
-
-def update_control_file(inf, outf, source_package_cb=None,
-                        binary_package_cb=None):
-    """Update a control file.
-
-    The callbacks can modify the paragraphs in place, and can trigger their
-    removal by clearing the paragraph.
-
-    Args:
-      inf: File-like object to read control file from
-      outf: File-like object to write control file to
-      source_package_cb: Called on source package paragraph (optional)
-      binary_package_cb: Called on each binary package paragraph (optional)
-    """
-    paragraphs = []
-    for paragraph in Deb822.iter_paragraphs(inf, encoding='utf-8'):
+def update_control(path='debian/control', source_package_cb=None,
+                   binary_package_cb=None):
+    def paragraph_cb(paragraph):
         if paragraph.get("Source"):
             if source_package_cb is not None:
                 source_package_cb(paragraph)
         else:
             if binary_package_cb is not None:
                 binary_package_cb(paragraph)
-        paragraphs.append(paragraph)
-    outf.write(dump_paragraphs(paragraphs))
+
+    return update_deb822(path, paragraph_cb=paragraph_cb)
 
 
 def parse_relations(text):
@@ -391,3 +315,20 @@ def ensure_minimum_debhelper_version(build_depends, minimum_version):
     return ensure_minimum_version(
             build_depends,
             "debhelper", minimum_version)
+
+
+def delete_from_list(liststr, item_to_delete):
+    items = liststr.split(',')
+    item_to_delete = item_to_delete.strip()
+    for i, item in enumerate(items):
+        if item.strip() == item_to_delete:
+            deleted_item = items.pop(i)
+            head_whitespace = ''.join(
+                takewhile(lambda x: x.isspace(), deleted_item))
+            if i == 0 and len(items) >= 1:
+                # If we're removing the first item, copy its whitespace to the
+                # second
+                items[i] = head_whitespace + items[i].lstrip()
+            elif i == len(items):
+                items[i-1] = items[i-1].rstrip()
+    return ','.join(items)
