@@ -59,6 +59,12 @@ class DetermineBrowserUrlTests(TestCase):
             determine_browser_url(
                 'https://salsa.debian.org/js-team/node-blah'))
 
+    def test_branch(self):
+        self.assertEqual(
+            'https://salsa.debian.org/js-team/node-blah/tree/debian',
+            determine_browser_url(
+                'https://salsa.debian.org/js-team/node-blah -b debian'))
+
 
 class SalsaUrlFromAliothUrlTests(TestCase):
 
@@ -78,14 +84,20 @@ class SalsaUrlFromAliothUrlTests(TestCase):
 
     def test_git(self):
         self.assertEqual(
-            'https://salsa.debian.org/python-team/modules/python-bla',
+            'https://salsa.debian.org/jelmer/python-bla',
             salsa_url_from_alioth_url(
                 'git',
-                'http://anonscm.debian.org/git/python-team/modules/python-bla')
+                'http://anonscm.debian.org/git/users/jelmer/python-bla')
             )
         self.assertEqual(
-            'https://salsa.debian.org/python-team/modules/python-bla',
+            'https://salsa.debian.org/jelmer/python-bla',
             salsa_url_from_alioth_url(
                 'git',
-                'http://anonscm.debian.org/python-team/modules/python-bla')
+                'http://anonscm.debian.org/users/jelmer/python-bla')
+            )
+        self.assertEqual(
+            'https://salsa.debian.org/go-team/golang-example-blah-blah',
+            salsa_url_from_alioth_url(
+                'git',
+                'http://anonscm.debian.org/pkg-go/golang-example-blah-blah')
             )
