@@ -17,16 +17,19 @@
 
 """Functions for working with upstream metadata."""
 
+import json
 import os
 import re
 import subprocess
 import tempfile
 from urllib.parse import urlparse
 from warnings import warn
+from lintian_brush import USER_AGENT
 from lintian_brush.vcs import (
     sanitize_url as sanitize_vcs_url,
     )
 from lintian_brush.watch import parse_watch_file
+from urllib.request import urlopen, Request
 
 
 KNOWN_HOSTING_SITES = [
@@ -34,6 +37,13 @@ KNOWN_HOSTING_SITES = [
     'salsa.debian.org']
 
 
+def get_sf_metadata(project):
+    headers = {'User-Agent': USER_AGENT}
+    http_url = 'https://sourceforge.net/rest/p/%s' % project
+    http_contents = urlopen(Request(http_url, headers=headers)).read()
+    return json.loads(http_contents)
+
+
 def guess_repo_from_url(url):
     parsed_url = urlparse(url)
     if parsed_url.netloc == 'github.com':
@@ -101,6 +111,11 @@ def guess_from_debian_watch(path, trust_package):
                 if repo:
                     yield "Repository", sanitize_vcs_url(repo), "possible"
                     break
+            m = re.match('https?://sf.net/([^/]+)', w.url)
+            if m:
+                yield "Archive", "SourceForge", "certain"
+                # TODO(jelmer): Yield sourceforge project name later
+                # so something later can call get_sf_metadata ?
 
 
 def guess_from_debian_control(path, trust_package):
@@ -160,6 +175,8 @@ def guess_from_package_json(path, trust_package):
                 # Some people seem to default github. :(
                 repo_url = 'https://github.com/' + parsed_url.path
                 yield 'Repository', sanitize_vcs_url(repo_url), 'possible'
+    if 'bugs' in package:
+        yield 'Bugs-Database', package['bugs'], 'certain'
 
 
 def guess_from_package_xml(path, trust_package):
@@ -245,13 +262,16 @@ def guess_from_debian_copyright(path, trust_package):
 
 
 def guess_from_readme(path, trust_package):
-    with open(path, 'rb') as f:
-        for line in f:
-            line = line.decode('utf-8', 'replace')
-            if line.strip().startswith('git clone'):
-                line = line.strip()
-                url = line.split()[2]
-                yield ('Repository', sanitize_vcs_url(url), 'possible')
+    try:
+        with open(path, 'rb') as f:
+            for line in f:
+                line = line.decode('utf-8', 'replace')
+                if line.strip().startswith('git clone'):
+                    line = line.strip()
+                    url = line.split()[2]
+                    yield ('Repository', sanitize_vcs_url(url), 'possible')
+    except IsADirectoryError:
+        pass
 
 
 def guess_from_meta_json(path, trust_package):
@@ -351,6 +371,46 @@ def guess_from_doap(path, trust_package):
                         yield 'Repository-Browse', web_url, 'certain'
 
 
+def guess_from_configure(path, trust_package=False):
+    with open(path, 'r') as f:
+        for line in f:
+            if '=' not in line:
+                continue
+            (key, value) = line.strip().split('=', 1)
+            if ' ' in key:
+                continue
+            if '$' in value:
+                continue
+            if value.startswith("'") and value.endswith("'"):
+                value = value[1:-1]
+            if key == 'PACKAGE_NAME':
+                yield 'Name', value, 'certain'
+            elif key == 'PACKAGE_BUGREPORT':
+                yield 'Bug-Submit', value, 'certain'
+            elif key == 'PACKAGE_URL':
+                yield 'Homepage', value, 'certain'
+
+
+def guess_from_r_description(path, trust_package=False):
+    with open(path, 'r') as f:
+        from debian.deb822 import Deb822
+        description = Deb822(f)
+        if 'Package' in description:
+            yield 'Name', description['Package'], 'certain'
+        if 'Repository' in description:
+            yield 'Archive', description['Repository'], 'certain'
+        if 'BugReports' in description:
+            yield 'Bug-Database', description['BugReports'], 'certain'
+        if 'URL' in description:
+            urls = [url.strip() for url in description['URL'].split(',')]
+            if len(urls) == 1:
+                yield 'Homepage', urls[0], 'possible'
+            for url in urls:
+                repo_url = guess_repo_from_url(url)
+                if repo_url:
+                    yield 'Repository', repo_url, 'certain'
+
+
 def guess_upstream_metadata_items(path, trust_package=False):
     """Guess upstream metadata items, in no particular order.
 
@@ -371,6 +431,8 @@ def guess_upstream_metadata_items(path, trust_package=False):
         ('debian/copyright', guess_from_debian_copyright),
         ('META.json', guess_from_meta_json),
         ('META.yml', guess_from_meta_yml),
+        ('configure', guess_from_configure),
+        ('DESCRIPTION', guess_from_r_description),
         ]
 
     doap_filenames = [n for n in os.listdir(path) if n.endswith('.doap')]
